//------------------------------------------------------------------------------
// Include the IRremote library header
//
#include <IRremote.h>
#include <HID-Project.h>  // HID-Project 2.8.2

bool sendHID = true;

// ATTENTION - set ALL debug and useSerial to false in normal usage
// as sending over Serial blocks the Arduino Leonardo unless a terminal application is connected!
bool debug = false;
bool debug2 = false;
bool useSerial = false;

#define BTN_1 1
#define BTN_2 2
#define BTN_3 3
#define BTN_4 4
#define BTN_5 5
#define BTN_6 6
#define BTN_7 7
#define BTN_8 8
#define BTN_9 9
#define BTN_0 10
#define BTN_PLAY 11
#define BTN_PAUSE 12
#define BTN_STOP 13
#define BTN_REC 14
#define BTN_REWD 15
#define BTN_FFWD 16
#define BTN_PREV 17
#define BTN_NEXT 18
#define BTN_LEFT 20
#define BTN_RIGHT 21
#define BTN_UP 22
#define BTN_DOWN 23
#define BTN_OK_ENTER 24
#define BTN_BACK 25
#define BTN_MENU 26
#define BTN_HOME 27
#define BTN_RED 30
#define BTN_GREEN 31
#define BTN_YELLOW 32
#define BTN_BLUE 33
#define BTN_STATUS 34
#define BTN_RETURN 35
#define BTN_SETUP 36
#define BTN_GUIDE 37
#define BTN_RADIO 38
#define BTN_PREVCH 39
#define BTN_CH_DOWN 40
#define BTN_CH_UP 41
#define BTN_VOL_DOWN 42
#define BTN_VOL_UP 43
#define BTN_MUTE 44
#define BTN_POWER 45
#define BTN_TV 46
#define BTN_VIDEOS 47
#define BTN_MUSIC 48
#define BTN_PICTURES 49
#define BTN_STAR 50
#define BTN_HASH 51
#define BTN_PAGE_UP 60
#define BTN_PAGE_DOWN 61
#define BTN_CONTEXT 62
#define BTN_INFO 63

#define BTN_DUMMY 255



unsigned long lastNECcode;
unsigned long lastReceivedMillis;


//------------------------------------------------------------------------------
// Tell IRremote which Arduino pin is connected to the IR Receiver (TSOP4838)
//
int recvPin = 2;
IRrecv irrecv(recvPin);

//+=============================================================================
// Configure the Arduino
//
void  setup ( )
{
  if(useSerial || debug || debug2) Serial.begin(115200);
  BootKeyboard.begin();
  Consumer.begin();
  System.begin();
  irrecv.enableIRIn();  // Start the receiver
}


//+=============================================================================
// Dump out the decode_results structure.
//
void  dumpInfo (decode_results *results)
{
  // Check if the buffer overflowed
  //  if (results->overflow) {
  //    Serial.println("IR code too long. Edit IRremoteInt.h and increase RAWLEN");
  //    return;
  //  }


  if (debug) {
    if ((results->decode_type != UNKNOWN && results->bits > 0) && debug2) {
      if (useSerial) {
        Serial.println();
        Serial.println();
        Serial.print(F("RECEIVED: "));
        encoding(results);
        Serial.print(";0x");
        ircode(results);
        Serial.print(";");
        Serial.print(results->bits, DEC);
        Serial.println();
      }
    }
  }

  // RC5 CODES
  if (results->decode_type == RC5 && results->bits == 12) {
    // RC5 code
    // 3 bytes, start byte for this device is 0xF or 0x7 depending on toggle bit

    uint8_t _currRC5Code = results->value ^ 0xF00;
    uint8_t _currRC5Pref = ( _currRC5Code ^ results->value )  >> 8;

    if (debug) {
      Serial.print(F("    _currRC5Code=0x"));
      Serial.print(_currRC5Code, HEX);
      Serial.print(F("   _currRC5Pref=0x"));
      Serial.println(_currRC5Pref, HEX);
    }


    // Hauppauge Remote
    if (_currRC5Code == 0xBD) { // POWER
      //handleButton_RCx(BTN_POWER, true, _currRC5Pref);
      handlePowerButton();
    }
    else if (_currRC5Code == 0x90) { // VOL+
      handleButton_RCx(BTN_VOL_UP, true, _currRC5Pref);
    }
    else if (_currRC5Code == 0x91) { // VOL-
      handleButton_RCx(BTN_VOL_DOWN, true, _currRC5Pref);
    }
    else if (_currRC5Code == 0xB2) { // REWIND
      handleButton_RCx(BTN_REWD, true, _currRC5Pref);
    }
    else if (_currRC5Code == 0xB4) { // FAST_FORWARD
      handleButton_RCx(BTN_FFWD, true, _currRC5Pref);
    }
    else if (_currRC5Code == 0xA1) { // CH- used as page down
      handleButton_RCx(BTN_CH_DOWN, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xA0) { // CH+ used as page up
      handleButton_RCx(BTN_CH_UP, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x94) { // ARROW UP
      handleButton_RCx(BTN_UP, true, _currRC5Pref);
    }
    else if (_currRC5Code == 0x95) { // ARROW DOWN
      handleButton_RCx(BTN_DOWN, true, _currRC5Pref);
    }
    else if (_currRC5Code == 0x96) { // ARROW LEFT
      handleButton_RCx(BTN_LEFT, true, _currRC5Pref);
    }
    else if (_currRC5Code == 0x97) { // ARROW RIGHT
      handleButton_RCx(BTN_RIGHT, true, _currRC5Pref);
    }
    else if (_currRC5Code == 0xA5) { // OK
      handleButton_RCx(BTN_OK_ENTER, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x9F) { // BACK/EXIT
      handleButton_RCx(BTN_BACK, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x8D) { // i/MENU
      handleButton_RCx(BTN_MENU, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x92) { // PREV.CH
      handleButton_RCx(BTN_CONTEXT, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x8F) { // MUTE
      handleButton_RCx(BTN_MUTE, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xBB) { // HOME/GO
      //handleButton_RCx(BTN_HOME, false, _currRC5Pref);
      handleTaskSwitchButton();
    }
    else if (_currRC5Code == 0x9C) { // TV
      handleButton_RCx(BTN_TV, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x98) { // VIDEOS
      handleButton_RCx(BTN_VIDEOS, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x99) { // MUSIC
      handleButton_RCx(BTN_MUSIC, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x9A) { // PICTURES
      handleButton_RCx(BTN_PICTURES, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x9B) { // GUIDE
      handleButton_RCx(BTN_GUIDE, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x8C) { // RADIO
      handleButton_RCx(BTN_RADIO, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xB7) { // REC
      handleButton_RCx(BTN_REC, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xB6) { // STOP
      handleButton_RCx(BTN_STOP, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xB5) { // PLAY
      handleButton_RCx(BTN_PLAY, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xB0) { // PAUSE
      handleButton_RCx(BTN_PAUSE, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xA4) { // PREV
      handleButton_RCx(BTN_PREV, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x9E) { // NEXT
      handleButton_RCx(BTN_NEXT, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x81) { // 1
      handleButton_RCx(BTN_1, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x82) { // 2
      handleButton_RCx(BTN_2, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x83) { // 3
      handleButton_RCx(BTN_3, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x84) { // 4
      handleButton_RCx(BTN_4, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x85) { // 5
      handleButton_RCx(BTN_5, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x86) { // 6
      handleButton_RCx(BTN_6, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x87) { // 7
      handleButton_RCx(BTN_7, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x88) { // 8
      handleButton_RCx(BTN_8, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x89) { // 9
      handleButton_RCx(BTN_9, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x80) { // 0
      handleButton_RCx(BTN_0, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x8A) { // *
      handleButton_RCx(BTN_STAR, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x8E) { // #
      handleButton_RCx(BTN_HASH, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0x8B) { // RED
      handleButton_RCx(BTN_RED, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xAE) { // GREEN
      handleButton_RCx(BTN_GREEN, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xB8) { // YELLOW
      handleButton_RCx(BTN_YELLOW, false, _currRC5Pref);
    }
    else if (_currRC5Code == 0xA9) { // BLUE
      handleButton_RCx(BTN_BLUE, false, _currRC5Pref);
    }

  } // END RC5 CODES



  // SONY CODES
  else if (results->decode_type == SONY && results->bits == 12) {

    // DENON RC-1014 REMOTE, Mode AUDIO/CD, MEDIA KEYS (set to SONY CD Player)
    if (results->value == 0x9D1) { // PAUSE
      handleButton(BTN_PAUSE, false);
    }
    else if (results->value == 0xCD1) { // REWD
      handleButton(BTN_REWD, true);
    }
    else if (results->value == 0x2D1) { // FFWD
      handleButton(BTN_FFWD, true);
    }
    else if (results->value == 0xD1) { // PREV
      handleButton(BTN_PREV, false);
    }
    else if (results->value == 0x8D1) { // NEXT
      handleButton(BTN_NEXT, false);
    }
    else if (results->value == 0x4D1) { // PLAY
      handleButton(BTN_PLAY, false);
    }
    else if (results->value == 0x1D1) { // STOP
      handleButton(BTN_STOP, false);
    }

  } // END SONY CODES




  // RC6 CODES
  else if (results->decode_type == RC6 && results->bits == 20) {
    unsigned int _currRC6Code = results->value ^ 0xF0000;
    uint8_t _currRC6Pref = ( _currRC6Code ^ results->value )  >> 16;

    if (debug) {
      Serial.print(F("    _currRC6Code=0x"));
      Serial.print(_currRC6Code, HEX);
      Serial.print(F("   _currRC6Pref=0x"));
      Serial.println(_currRC6Pref, HEX);
    }

    // Pioneer Remote in BD mode, configured code preset to Sat-PVR/Philips 6139 (RC6 code)
    // with the most important buttons available.
    // Missing buttons configured using learning function, which does not work well with RCx format due to the toggle bit,
    // so i used an old unused IR remote with NEC code for these, which will be OK for non-repeating buttons.
    if (_currRC6Code == 0x270C) { // POWER (of SOURCE)
      //handleButton_RCx(BTN_POWER, true, _currRC6Pref);
      handlePowerButton();
    }
    else if (_currRC6Code == 0x275C) { // OK/ENTER
      handleButton_RCx(BTN_OK_ENTER, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x275A) { // LEFT
      handleButton_RCx(BTN_LEFT, true, _currRC6Pref);
    }
    else if (_currRC6Code == 0x275B) { // RIGHT
      handleButton_RCx(BTN_RIGHT, true, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2758) { // UP
      handleButton_RCx(BTN_UP, true, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2759) { // DOWN
      handleButton_RCx(BTN_DOWN, true, _currRC6Pref);
    }
    else if (_currRC6Code == 0x27CC) { // HOME MENU - bottom left at control pad
      handleButton_RCx(BTN_MENU, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2783) { // RETURN/BACK - bottom right at control pad
      handleButton_RCx(BTN_BACK, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2743) { // X - AUDIO PARAMETER - top left at control pad
      //handleButton_RCx(BTN_HOME, false, _currRC6Pref);
      handleTaskSwitchButton();
    }
    else if (_currRC6Code == 0x2754) { // VIDEO PARAMETER - top right at control pad
      //handleButton_RCx(BTN_MENU, false, _currRC6Pref);
      handleButton_RCx(BTN_CONTEXT, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2771) { // PLAY
      handleButton_RCx(BTN_PLAY, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x276F) { // PAUSE
      handleButton_RCx(BTN_PAUSE, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x276E) { // STOP
      handleButton_RCx(BTN_STOP, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x276D) { // PREVIOUS
      handleButton_RCx(BTN_PREV, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2770) { // NEXT
      handleButton_RCx(BTN_NEXT, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2701) { // 1
      handleButton_RCx(BTN_1, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2702) { // 2
      handleButton_RCx(BTN_2, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2703) { // 3
      handleButton_RCx(BTN_3, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2704) { // 4
      handleButton_RCx(BTN_4, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2705) { // 5
      handleButton_RCx(BTN_5, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2706) { // 6
      handleButton_RCx(BTN_6, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2707) { // 7
      handleButton_RCx(BTN_7, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2708) { // 8
      handleButton_RCx(BTN_8, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2709) { // 9
      handleButton_RCx(BTN_9, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2700) { // 0
      handleButton_RCx(BTN_0, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2720) { // CH+
      handleButton_RCx(BTN_CH_UP, false, _currRC6Pref);
    }
    else if (_currRC6Code == 0x2721) { // CH-
      handleButton_RCx(BTN_CH_DOWN, false, _currRC6Pref);
    }

    // Buttons in NEC CODE due to limitations of this RC:
    //    RED
    //    GREEN
    //    YELLOW
    //    BLUE
    //    REWIND
    //    FAST FORWARD
    //    ./CLR
    //    CLASS/ENTER
    //    AUDIO
    //    INFO/DISP

  } // END RC6 CODES




  // NEC CODES - used also for Pioneer remote codes here (Pioneer code consists of 2 NEC codes and is not directly supported by Arduino IRRemote.h)
  else if (results->decode_type == NEC && results->bits == 32) {

    // Pioneer Remote in BD mode, configured code preset to Sat-PVR/Philips 6139 (RC6 code)
    // -> missing/unused buttons "learned" with codes in NEC format
    if (results->value == 0x4FF18E7) { // RED
      handleButton(BTN_RED, false);
    }
    else if (results->value == 0x4FF02FD) { // GREEN
      handleButton(BTN_GREEN, false);
    }
    else if (results->value == 0x4FF827D) { // YELLOW
      handleButton(BTN_YELLOW, false);
    }
    else if (results->value == 0x4FF38C7) { // BLUE
      handleButton(BTN_BLUE, false);
    }
    else if (results->value == 0x4FFB847) { // REWIND
      handleButton(BTN_REWD, false);
    }
    else if (results->value == 0x4FF08F7) { // FAST FORWARD
      handleButton(BTN_FFWD, false);
    }
    else if (results->value == 0x4FF6897) { // ./CLR
      //handleButton(BTN_, false);
    }
    else if (results->value == 0x4FF9867) { // CLASS/ENTER
      handleButton(BTN_OK_ENTER, false);
    }
    else if (results->value == 0x4FFF807) { // AUDIO
      //handleButton(BTN_, false);
    }
    else if (results->value == 0x4FF7887) { // DISPLAY/INFO
      handleButton(BTN_INFO, false);
    }




    // Pioneer Codes - here we have to take 2 received codes into account, so we check against the last received that we stored in a global variable then ;-)

    //    // Pioneer 2238 code for BD    ---- NOT UNUSED ANY MORE
    //    if (lastNECcode == 0xD52A34CB && results->value == 0xF50AC639) { // LEFT
    //      handleButton(BTN_LEFT, true);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A26D9) { // RIGHT
    //      handleButton(BTN_RIGHT, true);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A4FB0) { // UP
    //      handleButton(BTN_UP, true);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50ACF30) { // DOWN
    //      handleButton(BTN_DOWN, true);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50AF708) { // ENTER + CLASS ENTER
    //      handleButton(BTN_OK_ENTER, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A3DC2) { // POWER
    //      //handleButton(BTN_POWER, false);
    //      handlePowerButton();
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A2DD2) { // AUDIO PARAMETER - top left at control pad
    //      //handleButton(BTN_, false);
    //    }
    //    else if (lastNECcode == 0xD52A54AB && results->value == 0xF50A3CC3) { // VIDEO PARAMETER - top right at control pad
    //      //handleButton(BTN_, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A0DF2) { // HOME MENU - bottom left at control pad
    //      handleButton(BTN_MENU, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A2FD0) { // RETURN - bottom right at control pad
    //      handleButton(BTN_BACK, false);
    //    }
    //    else if (lastNECcode == 0xD52A54AB && results->value == 0xF50A26D9) { // RED
    //      handleButton(BTN_RED, false);
    //    }
    //    else if (lastNECcode == 0xD52A54AB && results->value == 0xF50AA659) { // GREEN
    //      handleButton(BTN_GREEN, false);
    //    }
    //    else if (lastNECcode == 0xD52A54AB && results->value == 0xF50A1FE0) { // YELLOW
    //      handleButton(BTN_YELLOW, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A9D62) { // BLUE
    //      handleButton(BTN_BLUE, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A57A8) { // REWIND
    //      handleButton(BTN_REWD, true);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A9768) { // FFWD
    //      handleButton(BTN_FFWD, true);
    //    }
    //    else if (lastNECcode == 0xD52A44BB && results->value == 0xD52A44BB) { // PLAY
    //      handleButton(BTN_PLAY, false);
    //    }
    //    else if (lastNECcode == 0xD52AE41B && results->value == 0xD52AE41B) { // PREVIOUS
    //      handleButton(BTN_PREV, false);
    //    }
    //    else if (lastNECcode == 0xD52AC43B && results->value == 0xD52AC43B) { // PAUSE
    //      handleButton(BTN_PAUSE, false);
    //    }
    //    else if (lastNECcode == 0xD52A04FB && results->value == 0xD52A04FB) { // STOP
    //      handleButton(BTN_STOP, false);
    //    }
    //    else if (lastNECcode == 0xD52A649B && results->value == 0xD52A649B) { // NEXT TRACK
    //      handleButton(BTN_NEXT, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A05FA) { // 0
    //      handleButton(BTN_0, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A857A) { // 1
    //      handleButton(BTN_1, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A45BA) { // 2
    //      handleButton(BTN_2, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50AC53A) { // 3
    //      handleButton(BTN_3, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A25DA) { // 4
    //      handleButton(BTN_4, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50AA55A) { // 5
    //      handleButton(BTN_5, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A659A) { // 6
    //      handleButton(BTN_6, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50AE51A) { // 7
    //      handleButton(BTN_7, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A15EA) { // 8
    //      handleButton(BTN_8, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A956A) { // 9
    //      handleButton(BTN_9, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50A7D82) { // AUDIO
    //      //handleButton(BTN_POWER, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50AC738) { // DISP/INFO
    //      //handleButton(BTN_POWER, false);
    //    }
    //    else if (lastNECcode == 0xD52A54AB && results->value == 0xF50A7689) { // CH_UP
    //      handleButton(BTN_CH_UP, false);
    //    }
    //    else if (lastNECcode == 0xD52A54AB && results->value == 0xF50AF609) { // CH_DOWN
    //      handleButton(BTN_CH_DOWN, false);
    //    }
    //    else if (lastNECcode == 0xD52A34CB && results->value == 0xF50AA758) { // ./CLR/D.ACCESS
    //      //handleButton(BTN_POWER, false);
    //    }

    // remember last NEC code
    lastNECcode = results->value;
  } // END NEC CODES


}


//+=============================================================================
// The repeating section of the code
//
void  loop ( )
{
  decode_results  results;        // Somewhere to store the results

  if (irrecv.decode(&results)) {  // Grab an IR code
    dumpInfo(&results);          // Output the results (if not in ignore time window)
    irrecv.resume();              // Prepare for the next value
  }

  handlePowerButton_loop();
  handleTaskSwitchButton_loop();

}
